@@ -117,24 +117,24 @@ static void fortranMangle(char *orig, char *mangled)
 {
   int uscore, i;
   
+  strcpy( mangled, orig );
   if(ELMER_LINKTYP == 1 || ELMER_LINKTYP == 3 || ELMER_LINKTYP == 4)
   {
-    for( i=0 ; i<strlen(orig) ; i++ ) /* to lower case */
+    for( i=0 ; i<strlen(mangled) ; i++ ) /* to lower case */
     {
-      if ( orig[i] >= 'A'  && orig[i] <= 'Z' ) 
-	orig[i] += 'a' - 'A';
+      if ( mangled[i] >= 'A'  && mangled[i] <= 'Z' ) 
+	mangled[i] += 'a' - 'A';
     }
   }
   if(ELMER_LINKTYP == 2)
   {
-    for( i=0; i<strlen(orig); i++ ) /* to upper case */
+    for( i=0; i<strlen(mangled); i++ ) /* to upper case */
     {
-      if ( orig[i] >= 'a'  && orig[i] <= 'z' ) 
-	orig[i] += 'A' - 'a';
+      if ( mangled[i] >= 'a'  && mangled[i] <= 'z' ) 
+	mangled[i] += 'A' - 'a';
     }
   }
   
-  strcpy( mangled, orig );
   if(ELMER_LINKTYP == 1) /* underscore */
   {
       strcat( mangled, "_" );
@@ -250,29 +250,81 @@ void *STDCALLBULL FC_FUNC(loadfunction,LOADFUNCTION) ( int *Quiet,
       fprintf( stderr, "Load: FATAL: Can't find procedure [%s]\n", NewName );
       exit(0);
    }
+
+
 #elif defined(HAVE_LOADLIBRARY_API)
 
-   if ( ( Handle = (void *)LoadLibrary( Library ) ) == NULL )
-   { 
-     fprintf( stderr, "Load: FATAL: Can't load shared image [%s]\n", Library );
+   /* Try again with explict ELMER_LIB dir */
+   ElmerLib[0] = '\0';
+   cptr = (char *)getenv( "ELMER_LIB" );
+   if ( cptr != NULL ) {
+      strncpy( ElmerLib, cptr, 2*MAX_NAME_LEN );
+      strncat( ElmerLib, "/", 2*MAX_NAME_LEN  );
+   } else {
+      cptr = (char *)getenv("ELMER_HOME");
+      if ( cptr != NULL  ) {
+         strncpy( ElmerLib, cptr, 2*MAX_NAME_LEN );
+         strncat( ElmerLib, "/share/elmersolver/lib/", 2*MAX_NAME_LEN );
+      } else {
+         strncpy( ElmerLib, ELMER_SOLVER_HOME, 2*MAX_NAME_LEN );
+         strncat( ElmerLib, "/lib/", 2*MAX_NAME_LEN );
+      }
+   }
 
-     /* try .dll */
-     strncat( NewLibName, SHL_EXTENSION, 3*MAX_NAME_LEN );
-     if ( ( Handle = (void *)LoadLibrary( NewLibName ) ) == NULL ) 
-     { 
-       fprintf( stderr, "Load: FATAL: Can't load shared image [%s]. Exiting.\n", NewLibName );
-       exit(0);
+   for( i=0; i<6; i++ )
+     {
+        switch(i) 
+        {
+   	  case 0: strncpy( NewLibName, Library,  3*MAX_NAME_LEN );
+                  break;
+          case 1: case 3: case 5:
+                  strncat( NewLibName, SHL_EXTENSION, 3*MAX_NAME_LEN );
+                  break;
+	  case 2: strcpy( NewLibName, "./");
+                  strncat( NewLibName, Library,  3*MAX_NAME_LEN );
+                  break;
+          case 4: strncpy( NewLibName, ElmerLib, 3*MAX_NAME_LEN );
+                  strncat( NewLibName, Library,  3*MAX_NAME_LEN );
+                  break;
+        }
+
+        if ( ( Handle = LoadLibrary( NewLibName ) ) == NULL )
+          {
+	    sprintf( dl_err_msg[i], "Can not find %s.", NewLibName );
+          } else {
+             break;
+          }
      }
-   }
 
-   for( i=0; i<strlen(Name); i++ )
-   {
-     if ( Name[i] >= 'a'  && Name[i] <= 'z' ) Name[i] += 'A' - 'a';
-   }
 
-   if ( (Function = (void *)GetProcAddress( Handle,Name ) ) == NULL )
+   if ( Handle == NULL ) 
+     {
+        for( i=0; i<6; i++ )
+          {
+             switch(i) 
+             {
+               case 0: strncpy( NewLibName, Library,  3*MAX_NAME_LEN );
+                       break;
+               case 1: case 3: case 5:
+                       strncat( NewLibName, SHL_EXTENSION, 3*MAX_NAME_LEN );
+                       break;
+               case 2: strcpy( NewLibName, "./");
+                       strncat( NewLibName, Library,  3*MAX_NAME_LEN );
+                       break;
+               case 4: strncpy( NewLibName, ElmerLib, 3*MAX_NAME_LEN );
+                       strncat( NewLibName, Library,  3*MAX_NAME_LEN );
+                       break;
+             }
+             fprintf( stderr, "\nLoad: Unable to open shared library: [%s]\n", NewLibName );
+             fprintf( stderr, "%s\n", dl_err_msg[i] );
+           }
+         exit(0);
+     }
+
+
+   if ( (Function = (void *)GetProcAddress( Handle,NewName ) ) == NULL )
    {
-     fprintf( stderr,"Load: FATAL: Can't find procedure [%s]\n", Name );
+     fprintf( stderr,"Load: FATAL: Can't find procedure [%s]\n", NewName );
      exit(0);
    }
 #endif 
@@ -457,8 +509,6 @@ static double DoViscFunction(double (STDCALLBULL *SolverProc)(), void *Model, vo
 double STDCALLBULL FC_FUNC(materialuserfunction,MATERIALUSERFUNCTION)
   ( f_ptr Function, void *Model, void *Element, void *Nodes, void *n, void *nd, void *Basis, void *GradBasis, void *Viscosity, void *Velo, void *gradV )
 {
-  fprintf(stderr,"entering materialuserfunc\n");
-  fflush(stderr);
    return DoViscFunction( (double (STDCALLBULL *)())*Function,Model,Element,Nodes,n,Basis,
                   GradBasis,Viscosity,Velo,gradV );
 }
